@@ -1,23 +1,18 @@
 use crate::auditwheel::PlatformTag;
-use crate::build_context::{BridgeModel, ProjectLayout};
+use crate::build_context::BridgeModel;
 use crate::cross_compile::{find_sysconfigdata, parse_sysconfigdata};
+use crate::project_layout::ProjectResolver;
 use crate::pyproject_toml::ToolMaturin;
 use crate::python_interpreter::{InterpreterConfig, InterpreterKind, MINIMUM_PYTHON_MINOR};
-use crate::BuildContext;
-use crate::CargoToml;
-use crate::Metadata21;
-use crate::PyProjectToml;
-use crate::PythonInterpreter;
-use crate::Target;
+use crate::{BuildContext, Metadata21, PythonInterpreter, Target};
 use anyhow::{bail, format_err, Context, Result};
-use cargo_metadata::{Metadata, MetadataCommand, Node};
+use cargo_metadata::{Metadata, Node};
 use regex::Regex;
 use serde::{Deserialize, Serialize};
 use std::collections::{HashMap, HashSet};
 use std::env;
-use std::io;
 use std::ops::{Deref, DerefMut};
-use std::path::{Path, PathBuf};
+use std::path::PathBuf;
 
 // This is used for BridgeModel::Bindings("pyo3-ffi") and BridgeModel::Bindings("pyo3").
 // These should be treated almost identically but must be correctly identified
@@ -142,8 +137,9 @@ pub struct BuildOptions {
     /// or `musllinux` tags (for example `musllinux_1_2`)
     /// and `linux` for the native linux tag.
     ///
-    /// Note that `manylinux1` is unsupported by the rust compiler. Wheels with the native `linux` tag
-    /// will be rejected by pypi, unless they are separately validated by `auditwheel`.
+    /// Note that `manylinux1` and `manylinux2010` is unsupported by the rust compiler.
+    /// Wheels with the native `linux` tag will be rejected by pypi,
+    /// unless they are separately validated by `auditwheel`.
     ///
     /// The default is the lowest compatible `manylinux` tag, or plain `linux` if nothing matched
     ///
@@ -159,7 +155,7 @@ pub struct BuildOptions {
     pub platform_tag: Vec<PlatformTag>,
 
     /// The python versions to build wheels for, given as the names of the
-    /// interpreters. Uses autodiscovery if not explicitly set.
+    /// interpreters.
     #[clap(short, long, multiple_values = true, multiple_occurrences = true)]
     pub interpreter: Vec<PathBuf>,
 
@@ -182,7 +178,7 @@ pub struct BuildOptions {
 
     /// For manylinux targets, use zig to ensure compliance for the chosen manylinux version
     ///
-    /// Default to manylinux2010/manylinux_2_12 if you do not specify an `--compatibility`
+    /// Default to manylinux2014/manylinux_2_17 if you do not specify an `--compatibility`
     ///
     /// Make sure you installed zig with `pip install maturin[zig]`
     #[clap(long)]
@@ -213,45 +209,6 @@ impl DerefMut for BuildOptions {
 }
 
 impl BuildOptions {
-    /// Get cargo manifest file path
-    fn manifest_path(&self) -> Result<PathBuf> {
-        // use command line argument if specified
-        if let Some(path) = &self.manifest_path {
-            return Ok(path.clone());
-        }
-        // check `manifest-path` option in pyproject.toml
-        let current_dir = env::current_dir()
-            .context("Failed to detect current directory ಠ_ಠ")?
-            .canonicalize()?;
-        if current_dir.join("pyproject.toml").is_file() {
-            let pyproject =
-                PyProjectToml::new(&current_dir).context("pyproject.toml is invalid")?;
-            if let Some(path) = pyproject.manifest_path() {
-                println!("🔗 Found cargo manifest path in pyproject.toml");
-                // pyproject.toml must be placed at top directory
-                let manifest_dir = path
-                    .parent()
-                    .context("missing parent directory")?
-                    .canonicalize()?;
-                if !manifest_dir.starts_with(&current_dir) {
-                    bail!("Cargo.toml can not be placed outside of the directory containing pyproject.toml");
-                }
-                return Ok(path.to_path_buf());
-            }
-        }
-        // check Cargo.toml in current directory
-        let path = PathBuf::from("Cargo.toml");
-        if path.exists() {
-            Ok(path)
-        } else {
-            Err(format_err!(
-                "Can't find {} (in {})",
-                path.display(),
-                current_dir.display()
-            ))
-        }
-    }
-
     /// Finds the appropriate amount for python versions for each [BridgeModel].
     fn find_interpreters(
         &self,
@@ -446,18 +403,28 @@ impl BuildOptions {
                         bail!("Failed to find a python interpreter");
                     }
                 } else {
-                    let interpreters = find_interpreter_in_host(
+                    let found_interpreters = find_interpreter_in_host(
                         bridge,
                         interpreter,
                         target,
                         Some(*minor as usize),
                     )
-                    .unwrap_or_else(|_| {
-                        find_interpreter_in_sysconfig(interpreter, target, Some(*minor as usize))
-                            .unwrap_or_default()
-                    });
+                    .or_else(|err| {
+                        let interps = find_interpreter_in_sysconfig(
+                            interpreter,
+                            target,
+                            Some(*minor as usize),
+                        )
+                        .unwrap_or_default();
+                        if interps.is_empty() && !self.interpreter.is_empty() {
+                            // Print error when user supplied `--interpreter` option
+                            Err(err)
+                        } else {
+                            Ok(interps)
+                        }
+                    })?;
                     println!("🐍 Not using a specific python interpreter");
-                    if interpreter.is_empty() {
+                    if self.interpreter.is_empty() {
                         // Fake one to make `BuildContext::build_wheels` happy for abi3 when no cpython/pypy found on host
                         // The python interpreter config doesn't matter, as it's not used for anything
                         Ok(vec![PythonInterpreter {
@@ -475,9 +442,9 @@ impl BuildOptions {
                             runnable: false,
                         }])
                     } else if target.cross_compiling() {
-                        let mut interps = Vec::with_capacity(interpreters.len());
+                        let mut interps = Vec::with_capacity(found_interpreters.len());
                         let mut pypys = Vec::new();
-                        for interp in interpreters {
+                        for interp in found_interpreters {
                             if interp.interpreter_kind.is_pypy() {
                                 pypys.push(PathBuf::from(format!(
                                     "pypy{}.{}",
@@ -501,10 +468,10 @@ impl BuildOptions {
                         }
                         Ok(interps)
                     } else {
-                        if interpreters.is_empty() {
+                        if found_interpreters.is_empty() {
                             bail!("Failed to find any python interpreter");
                         }
-                        Ok(interpreters)
+                        Ok(found_interpreters)
                     }
                 }
             }
@@ -518,94 +485,26 @@ impl BuildOptions {
         strip: bool,
         editable: bool,
     ) -> Result<BuildContext> {
-        let manifest_file = self.manifest_path()?;
-        if !manifest_file.is_file() {
-            bail!(
-                "{} is not the path to a Cargo.toml",
-                manifest_file.display()
-            );
-        }
-
-        let cargo_toml = CargoToml::from_path(&manifest_file)?;
-        let manifest_dir = manifest_file.parent().unwrap();
-        let pyproject: Option<PyProjectToml> = if manifest_dir.join("pyproject.toml").is_file() {
-            let pyproject =
-                PyProjectToml::new(manifest_dir).context("pyproject.toml is invalid")?;
-            pyproject.warn_missing_maturin_version();
-            pyproject.warn_missing_build_backend();
-            Some(pyproject)
-        } else {
-            None
-        };
-        let pyproject = pyproject.as_ref();
-        let tool_maturin = pyproject.and_then(|p| p.maturin());
-
-        let metadata21 = Metadata21::from_cargo_toml(&cargo_toml, &manifest_dir)
-            .context("Failed to parse Cargo.toml into python metadata")?;
-        let extra_metadata = cargo_toml.remaining_core_metadata();
-
-        let crate_name = &cargo_toml.package.name;
-
-        // If the package name contains minuses, you must declare a module with
-        // underscores as lib name
-        let module_name = cargo_toml
-            .lib
-            .as_ref()
-            .and_then(|lib| lib.name.as_ref())
-            .unwrap_or(crate_name)
-            .to_owned();
-
-        // Only use extension name from extra metadata if it contains dot
-        let extension_name = extra_metadata
-            .name
-            .as_ref()
-            .filter(|name| name.contains('.'))
-            .unwrap_or(&module_name);
-
-        let data = pyproject
-            .and_then(|x| x.data())
-            .or_else(|| extra_metadata.data.as_ref().map(Path::new));
-        let project_layout = ProjectLayout::determine(
-            manifest_dir,
-            extension_name,
-            extra_metadata.python_source.as_deref(),
-            data,
-        )?;
-
-        let mut cargo_options = self.cargo.clone();
-
-        let mut args_from_pyproject = if let Some(tool_maturin) = tool_maturin {
-            cargo_options.merge_with_pyproject_toml(tool_maturin.clone())
-        } else {
-            Vec::new()
-        };
-
-        let cargo_metadata_extra_args = extract_cargo_metadata_args(&self.cargo)?;
-
-        let result = MetadataCommand::new()
-            .manifest_path(&manifest_file)
-            .other_options(cargo_metadata_extra_args)
-            .exec();
-
-        let cargo_metadata = match result {
-            Ok(cargo_metadata) => cargo_metadata,
-            Err(cargo_metadata::Error::Io(inner)) if inner.kind() == io::ErrorKind::NotFound => {
-                // NotFound is the specific error when cargo is not in PATH
-                return Err(inner)
-                    .context("Cargo metadata failed. Do you have cargo in your PATH?");
-            }
-            Err(err) => {
-                return Err(err)
-                    .context("Cargo metadata failed. Does your crate compile with `cargo build`?");
-            }
-        };
+        let ProjectResolver {
+            project_layout,
+            cargo_toml_path,
+            cargo_toml,
+            pyproject_toml_path,
+            pyproject_toml,
+            module_name,
+            metadata21,
+            mut cargo_options,
+            cargo_metadata,
+            mut pyproject_toml_maturin_options,
+        } = ProjectResolver::resolve(self.manifest_path.clone(), self.cargo.clone())?;
+        let pyproject = pyproject_toml.as_ref();
 
         let bridge = find_bridge(
             &cargo_metadata,
             self.bindings.as_deref().or_else(|| {
                 pyproject.and_then(|x| {
                     if x.bindings().is_some() {
-                        args_from_pyproject.push("bindings");
+                        pyproject_toml_maturin_options.push("bindings");
                     }
                     x.bindings()
                 })
@@ -676,9 +575,10 @@ impl BuildOptions {
 
         if cargo_options.args.is_empty() {
             // if not supplied on command line, try pyproject.toml
+            let tool_maturin = pyproject.and_then(|p| p.maturin());
             if let Some(args) = tool_maturin.and_then(|x| x.rustc_args.as_ref()) {
                 cargo_options.args.extend(args.iter().cloned());
-                args_from_pyproject.push("rustc-args");
+                pyproject_toml_maturin_options.push("rustc-args");
             }
         }
 
@@ -689,7 +589,7 @@ impl BuildOptions {
             let compatibility = pyproject
                 .and_then(|x| {
                     if x.compatibility().is_some() {
-                        args_from_pyproject.push("compatibility");
+                        pyproject_toml_maturin_options.push("compatibility");
                     }
                     x.compatibility()
                 })
@@ -718,11 +618,14 @@ impl BuildOptions {
         } else {
             self.platform_tag
         };
-        if platform_tags
-            .iter()
-            .any(|tag| tag == &PlatformTag::manylinux1())
-        {
-            eprintln!("⚠️  Warning: manylinux1 is unsupported by the Rust compiler.");
+
+        for platform_tag in &platform_tags {
+            if !platform_tag.is_supported() {
+                eprintln!(
+                    "⚠️  Warning: {} is unsupported by the Rust compiler.",
+                    platform_tag
+                );
+            }
         }
 
         match bridge {
@@ -770,10 +673,10 @@ impl BuildOptions {
             bail!("Cannot mix linux and manylinux/musllinux platform tags",);
         }
 
-        if !args_from_pyproject.is_empty() {
+        if !pyproject_toml_maturin_options.is_empty() {
             eprintln!(
                 "📡 Using build options {} from pyproject.toml",
-                args_from_pyproject.join(", ")
+                pyproject_toml_maturin_options.join(", ")
             );
         }
 
@@ -782,15 +685,18 @@ impl BuildOptions {
             .target_dir
             .clone()
             .unwrap_or_else(|| cargo_metadata.target_directory.clone().into_std_path_buf());
+        let crate_name = cargo_toml.package.name;
 
         Ok(BuildContext {
             target,
             bridge,
             project_layout,
+            pyproject_toml_path,
+            pyproject_toml,
             metadata21,
-            crate_name: crate_name.to_string(),
+            crate_name,
             module_name,
-            manifest_path: manifest_file,
+            manifest_path: cargo_toml_path,
             target_dir,
             out: wheel_dir,
             release,
@@ -1138,7 +1044,7 @@ fn find_interpreter_in_sysconfig(
 /// in the same string as its name or in the next one. For this naive parsing logic, we
 /// assume that the value is in the next argument if the argument string equals the name,
 /// otherwise it's in the same argument and the next argument is unrelated.
-fn extract_cargo_metadata_args(cargo_options: &CargoOptions) -> Result<Vec<String>> {
+pub(crate) fn extract_cargo_metadata_args(cargo_options: &CargoOptions) -> Result<Vec<String>> {
     let mut cargo_metadata_extra_args = vec![];
     if cargo_options.frozen {
         cargo_metadata_extra_args.push("--frozen".to_string());
@@ -1201,9 +1107,15 @@ impl From<CargoOptions> for cargo_options::Rustc {
 }
 
 impl CargoOptions {
-    fn merge_with_pyproject_toml(&mut self, tool_maturin: ToolMaturin) -> Vec<&'static str> {
+    /// Merge options from pyproject.toml
+    pub fn merge_with_pyproject_toml(&mut self, tool_maturin: ToolMaturin) -> Vec<&'static str> {
         let mut args_from_pyproject = Vec::new();
 
+        if self.manifest_path.is_none() && tool_maturin.manifest_path.is_some() {
+            self.manifest_path = tool_maturin.manifest_path.clone();
+            args_from_pyproject.push("manifest-path");
+        }
+
         if self.profile.is_none() && tool_maturin.profile.is_some() {
             self.profile = tool_maturin.profile.clone();
             args_from_pyproject.push("profile");
@@ -1264,6 +1176,8 @@ impl CargoOptions {
 
 #[cfg(test)]
 mod test {
+    use cargo_metadata::MetadataCommand;
+    use pretty_assertions::assert_eq;
     use std::path::Path;
 
     use super::*;
@@ -1422,10 +1336,18 @@ mod test {
 
     #[test]
     fn test_get_min_python_minor() {
+        use crate::CargoToml;
+
         // Nothing specified
-        let cargo_toml = CargoToml::from_path("test-crates/pyo3-pure/Cargo.toml").unwrap();
+        let manifest_path = "test-crates/pyo3-pure/Cargo.toml";
+        let cargo_toml = CargoToml::from_path(manifest_path).unwrap();
+        let cargo_metadata = MetadataCommand::new()
+            .manifest_path(manifest_path)
+            .exec()
+            .unwrap();
         let metadata21 =
-            Metadata21::from_cargo_toml(&cargo_toml, &"test-crates/pyo3-pure").unwrap();
+            Metadata21::from_cargo_toml(&cargo_toml, &"test-crates/pyo3-pure", &cargo_metadata)
+                .unwrap();
         assert_eq!(get_min_python_minor(&metadata21), None);
     }
 }
