@@ -28,7 +28,7 @@
 pub use crate::build_context::{BridgeModel, BuildContext, BuiltWheelMetadata};
 pub use crate::build_options::{BuildOptions, CargoOptions};
 pub use crate::cargo_toml::CargoToml;
-pub use crate::compile::compile;
+pub use crate::compile::{compile, BuildArtifact};
 pub use crate::develop::develop;
 pub use crate::metadata::{Metadata21, WheelMetadata};
 pub use crate::module_writer::{
@@ -52,6 +52,7 @@ mod develop;
 mod metadata;
 mod module_writer;
 mod new_project;
+mod project_layout;
 mod pyproject_toml;
 mod python_interpreter;
 mod source_distribution;
