@@ -62,13 +62,13 @@ enum Opt {
         build: BuildOptions,
     },
     #[clap(name = "list-python")]
-    /// Searches and lists the available python installations
+    /// Search and list the available python installations
     ListPython {
         #[clap(long)]
         target: Option<String>,
     },
     #[clap(name = "develop", alias = "dev")]
-    /// Installs the crate as module in the current virtualenv
+    /// Install the crate as module in the current virtualenv
     ///
     /// Note that this command doesn't create entrypoints
     Develop {
@@ -102,14 +102,9 @@ enum Opt {
     /// This command is a workaround for [pypa/pip#6041](https://github.com/pypa/pip/issues/6041)
     #[clap(name = "sdist")]
     SDist {
-        #[clap(
-            short = 'm',
-            long = "manifest-path",
-            parse(from_os_str),
-            default_value = "Cargo.toml"
-        )]
+        #[clap(short = 'm', long = "manifest-path", parse(from_os_str))]
         /// The path to the Cargo.toml
-        manifest_path: PathBuf,
+        manifest_path: Option<PathBuf>,
         /// The directory to store the built wheels in. Defaults to a new "wheels"
         /// directory in the project's target directory
         #[clap(short, long, parse(from_os_str))]
@@ -131,7 +126,7 @@ enum Opt {
         #[clap(flatten)]
         options: GenerateProjectOptions,
     },
-    /// Uploads python packages to pypi
+    /// Upload python packages to pypi
     ///
     /// It is mostly similar to `twine upload`, but can only upload python wheels
     /// and source distributions.
@@ -302,7 +297,7 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
             let build_context = build_options.into_build_context(false, false, false)?;
             let (path, _) = build_context
                 .build_source_distribution()?
-                .context("Failed to build source distribution")?;
+                .context("Failed to build source distribution, pyproject.toml not found")?;
             println!("{}", path.file_name().unwrap().to_str().unwrap());
         }
     };
@@ -314,6 +309,19 @@ fn run() -> Result<()> {
     #[cfg(feature = "log")]
     pretty_env_logger::init();
 
+    // Allow symlink `maturin` to `ar` to invoke `zig ar`
+    // See https://github.com/messense/cargo-zigbuild/issues/52
+    let mut args = env::args();
+    let program_path = PathBuf::from(args.next().expect("no program path"));
+    let program_name = program_path.file_stem().expect("no program name");
+    if program_name.eq_ignore_ascii_case("ar") {
+        let zig = Zig::Ar {
+            args: args.collect(),
+        };
+        zig.execute()?;
+        return Ok(());
+    }
+
     let opt = Opt::parse();
 
     match opt {
@@ -325,7 +333,9 @@ fn run() -> Result<()> {
         } => {
             let build_context = build.into_build_context(release, strip, false)?;
             if sdist {
-                build_context.build_source_distribution()?;
+                build_context
+                    .build_source_distribution()?
+                    .context("Failed to build source distribution, pyproject.toml not found")?;
             }
             let wheels = build_context.build_wheels()?;
             assert!(!wheels.is_empty());
@@ -398,7 +408,7 @@ fn run() -> Result<()> {
             let build_options = BuildOptions {
                 out,
                 cargo: CargoOptions {
-                    manifest_path: Some(manifest_path),
+                    manifest_path,
                     ..Default::default()
                 },
                 ..Default::default()
@@ -406,7 +416,7 @@ fn run() -> Result<()> {
             let build_context = build_options.into_build_context(false, false, false)?;
             build_context
                 .build_source_distribution()?
-                .context("Failed to build source distribution")?;
+                .context("Failed to build source distribution, pyproject.toml not found")?;
         }
         Opt::Pep517(subcommand) => pep517(subcommand)?,
         Opt::InitProject { path, options } => init_project(path, options)?,
