@@ -1,9 +1,6 @@
 //! The wheel format is (mostly) specified in PEP 427
-
-use crate::build_context::ProjectLayout;
-use crate::PythonInterpreter;
-use crate::Target;
-use crate::{BridgeModel, Metadata21};
+use crate::project_layout::ProjectLayout;
+use crate::{BridgeModel, Metadata21, PythonInterpreter, Target};
 use anyhow::{anyhow, bail, Context, Result};
 use flate2::write::GzEncoder;
 use flate2::Compression;
@@ -277,7 +274,7 @@ impl WheelWriter {
         metadata21: &Metadata21,
     ) -> Result<()> {
         if let Some(python_module) = &project_layout.python_module {
-            let absolute_path = python_module.canonicalize()?;
+            let absolute_path = fs::canonicalize(python_module)?;
             if let Some(python_path) = absolute_path.parent().and_then(|p| p.to_str()) {
                 let name = metadata21.get_distribution_escaped();
                 self.add_bytes(format!("{}.pth", name), python_path.as_bytes())?;
