@@ -1,8 +1,8 @@
 use crate::PlatformTag;
-use anyhow::{format_err, Context, Result};
+use anyhow::{format_err, Result};
+use fs_err as fs;
 use pyproject_toml::PyProjectToml as ProjectToml;
 use serde::{Deserialize, Serialize};
-use std::fs;
 use std::path::{Path, PathBuf};
 
 /// The `[tool]` section of a pyproject.toml
@@ -25,6 +25,8 @@ pub struct ToolMaturin {
     skip_auditwheel: bool,
     #[serde(default)]
     strip: bool,
+    /// The directory with python module, contains `<module_name>/__init__.py`
+    python_source: Option<PathBuf>,
     /// Path to the wheel directory, defaults to `<module_name>.data`
     data: Option<PathBuf>,
     // Some customizable cargo options
@@ -76,12 +78,9 @@ impl PyProjectToml {
     ///
     /// Does no specific error handling because it's only used to check whether or not to build
     /// source distributions
-    pub fn new(project_root: impl AsRef<Path>) -> Result<PyProjectToml> {
-        let path = project_root.as_ref().join("pyproject.toml");
-        let contents = fs::read_to_string(&path).context(format!(
-            "Couldn't find pyproject.toml at {}",
-            path.display()
-        ))?;
+    pub fn new(pyproject_file: impl AsRef<Path>) -> Result<PyProjectToml> {
+        let path = pyproject_file.as_ref();
+        let contents = fs::read_to_string(&path)?;
         let pyproject: PyProjectToml = toml_edit::easy::from_str(&contents)
             .map_err(|err| format_err!("pyproject.toml is not PEP 517 compliant: {}", err))?;
         Ok(pyproject)
@@ -122,6 +121,12 @@ impl PyProjectToml {
             .unwrap_or_default()
     }
 
+    /// Returns the value of `[tool.maturin.python-source]` in pyproject.toml
+    pub fn python_source(&self) -> Option<&Path> {
+        self.maturin()
+            .and_then(|maturin| maturin.python_source.as_deref())
+    }
+
     /// Returns the value of `[tool.maturin.data]` in pyproject.toml
     pub fn data(&self) -> Option<&Path> {
         self.maturin().and_then(|maturin| maturin.data.as_deref())
@@ -185,15 +190,17 @@ impl PyProjectToml {
 mod tests {
     use crate::PyProjectToml;
     use fs_err as fs;
+    use pretty_assertions::assert_eq;
     use std::path::Path;
     use tempfile::TempDir;
 
     #[test]
     fn test_parse_tool_maturin() {
         let tmp_dir = TempDir::new().unwrap();
+        let pyproject_file = tmp_dir.path().join("pyproject.toml");
 
         fs::write(
-            tmp_dir.path().join("pyproject.toml"),
+            &pyproject_file,
             r#"[build-system]
             requires = ["maturin"]
             build-backend = "maturin"
@@ -209,7 +216,7 @@ mod tests {
             "#,
         )
         .unwrap();
-        let pyproject = PyProjectToml::new(tmp_dir).unwrap();
+        let pyproject = PyProjectToml::new(pyproject_file).unwrap();
         assert_eq!(pyproject.manifest_path(), Some(Path::new("Cargo.toml")));
 
         let maturin = pyproject.maturin().unwrap();
@@ -230,12 +237,13 @@ mod tests {
 
     #[test]
     fn test_warn_missing_maturin_version() {
-        let with_constraint = PyProjectToml::new("test-crates/pyo3-pure").unwrap();
+        let with_constraint = PyProjectToml::new("test-crates/pyo3-pure/pyproject.toml").unwrap();
         assert!(with_constraint.warn_missing_maturin_version());
         let without_constraint_dir = TempDir::new().unwrap();
+        let pyproject_file = without_constraint_dir.path().join("pyproject.toml");
 
         fs::write(
-            without_constraint_dir.path().join("pyproject.toml"),
+            &pyproject_file,
             r#"[build-system]
             requires = ["maturin"]
             build-backend = "maturin"
@@ -245,7 +253,7 @@ mod tests {
             "#,
         )
         .unwrap();
-        let without_constraint = PyProjectToml::new(without_constraint_dir).unwrap();
+        let without_constraint = PyProjectToml::new(pyproject_file).unwrap();
         assert!(!without_constraint.warn_missing_maturin_version());
     }
 }
