@@ -3,6 +3,7 @@ use crate::python_interpreter::InterpreterKind;
 use crate::{PlatformTag, PythonInterpreter};
 use anyhow::{anyhow, bail, format_err, Context, Result};
 use platform_info::*;
+use rustc_version::VersionMeta;
 use serde::Deserialize;
 use std::env;
 use std::fmt;
@@ -11,6 +12,8 @@ use std::path::PathBuf;
 use std::str;
 use target_lexicon::{Environment, Triple};
 
+pub(crate) const RUST_1_64_0: semver::Version = semver::Version::new(1, 64, 0);
+
 /// All supported operating system
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash, Deserialize)]
 #[serde(rename_all = "lowercase")]
@@ -51,6 +54,8 @@ pub enum Arch {
     Aarch64,
     Armv6L,
     Armv7L,
+    #[serde(alias = "ppc")]
+    Powerpc,
     #[serde(alias = "ppc64le")]
     Powerpc64Le,
     #[serde(alias = "ppc64")]
@@ -60,6 +65,10 @@ pub enum Arch {
     X86_64,
     S390X,
     Wasm32,
+    Riscv64,
+    Mips64el,
+    Mipsel,
+    Sparc64,
 }
 
 impl fmt::Display for Arch {
@@ -68,12 +77,17 @@ impl fmt::Display for Arch {
             Arch::Aarch64 => write!(f, "aarch64"),
             Arch::Armv6L => write!(f, "armv6l"),
             Arch::Armv7L => write!(f, "armv7l"),
+            Arch::Powerpc => write!(f, "ppc"),
             Arch::Powerpc64Le => write!(f, "ppc64le"),
             Arch::Powerpc64 => write!(f, "ppc64"),
             Arch::X86 => write!(f, "i686"),
             Arch::X86_64 => write!(f, "x86_64"),
             Arch::S390X => write!(f, "s390x"),
             Arch::Wasm32 => write!(f, "wasm32"),
+            Arch::Riscv64 => write!(f, "riscv64"),
+            Arch::Mips64el => write!(f, "mips64el"),
+            Arch::Mipsel => write!(f, "mipsel"),
+            Arch::Sparc64 => write!(f, "sparc64"),
         }
     }
 }
@@ -85,11 +99,16 @@ fn get_supported_architectures(os: &Os) -> Vec<Arch> {
             Arch::Aarch64,
             Arch::Armv6L,
             Arch::Armv7L,
+            Arch::Powerpc,
             Arch::Powerpc64,
             Arch::Powerpc64Le,
             Arch::S390X,
             Arch::X86,
             Arch::X86_64,
+            Arch::Riscv64,
+            Arch::Mips64el,
+            Arch::Mipsel,
+            Arch::Sparc64,
         ],
         Os::Windows => vec![Arch::X86, Arch::X86_64, Arch::Aarch64],
         Os::Macos => vec![Arch::Aarch64, Arch::X86_64],
@@ -116,8 +135,8 @@ pub struct Target {
     env: Environment,
     triple: String,
     cross_compiling: bool,
-    /// Host machine target triple
-    pub(crate) host_triple: String,
+    /// rustc version information
+    pub(crate) rustc_version: VersionMeta,
     /// Is user specified `--target`
     pub(crate) user_specified: bool,
 }
@@ -128,9 +147,12 @@ impl Target {
     ///
     /// Fails if the target triple isn't supported
     pub fn from_target_triple(target_triple: Option<String>) -> Result<Self> {
-        use target_lexicon::{Architecture, ArmArchitecture, OperatingSystem};
+        use target_lexicon::{
+            Architecture, ArmArchitecture, Mips32Architecture, Mips64Architecture, OperatingSystem,
+        };
 
-        let host_triple = get_host_target()?;
+        let rustc_version = rustc_version_meta()?;
+        let host_triple = &rustc_version.host;
         let (platform, triple) = if let Some(ref target_triple) = target_triple {
             let platform: Triple = target_triple
                 .parse()
@@ -165,10 +187,15 @@ impl Target {
                 _ => Arch::Armv7L,
             },
             Architecture::Aarch64(_) => Arch::Aarch64,
+            Architecture::Powerpc => Arch::Powerpc,
             Architecture::Powerpc64 => Arch::Powerpc64,
             Architecture::Powerpc64le => Arch::Powerpc64Le,
             Architecture::S390x => Arch::S390X,
             Architecture::Wasm32 => Arch::Wasm32,
+            Architecture::Riscv64(_) => Arch::Riscv64,
+            Architecture::Mips64(Mips64Architecture::Mips64el) => Arch::Mips64el,
+            Architecture::Mips32(Mips32Architecture::Mipsel) => Arch::Mipsel,
+            Architecture::Sparc64 => Arch::Sparc64,
             unsupported => bail!("The architecture {} is not supported", unsupported),
         };
 
@@ -181,7 +208,7 @@ impl Target {
             arch,
             env: platform.environment,
             triple,
-            host_triple,
+            rustc_version,
             user_specified: target_triple.is_some(),
             cross_compiling: false,
         };
@@ -351,12 +378,18 @@ impl Target {
             Arch::Aarch64 => "aarch64",
             Arch::Armv6L => "armv6l",
             Arch::Armv7L => "armv7l",
+            Arch::Powerpc => "ppc",
             Arch::Powerpc64Le => "powerpc64le",
             Arch::Powerpc64 => "powerpc64",
             Arch::X86 => "i386",
             Arch::X86_64 => "x86_64",
             Arch::S390X => "s390x",
             Arch::Wasm32 => "wasm32",
+            Arch::Riscv64 => "riscv64",
+            // It's kinda surprising that Python doesn't include the `el` suffix
+            Arch::Mips64el => "mips64",
+            Arch::Mipsel => "mips",
+            Arch::Sparc64 => "sparc64",
         }
     }
 
@@ -382,24 +415,55 @@ impl Target {
             Arch::Aarch64 | Arch::Armv7L | Arch::Powerpc64 | Arch::Powerpc64Le | Arch::S390X => {
                 PlatformTag::manylinux2014()
             }
-            Arch::X86 | Arch::X86_64 => PlatformTag::manylinux2010(),
-            Arch::Armv6L | Arch::Wasm32 => PlatformTag::Linux,
+            Arch::X86 | Arch::X86_64 => {
+                // rustc 1.64.0 bumps glibc requirement to 2.17
+                // see https://blog.rust-lang.org/2022/08/01/Increasing-glibc-kernel-requirements.html
+                if self.rustc_version.semver >= RUST_1_64_0 {
+                    PlatformTag::manylinux2014()
+                } else {
+                    PlatformTag::manylinux2010()
+                }
+            }
+            Arch::Armv6L
+            | Arch::Wasm32
+            | Arch::Riscv64
+            | Arch::Mips64el
+            | Arch::Mipsel
+            | Arch::Powerpc
+            | Arch::Sparc64 => PlatformTag::Linux,
         }
     }
 
     /// Returns whether the platform is 64 bit or 32 bit
     pub fn pointer_width(&self) -> usize {
         match self.arch {
-            Arch::Aarch64 | Arch::Powerpc64 | Arch::Powerpc64Le | Arch::X86_64 | Arch::S390X => 64,
-            Arch::Armv6L | Arch::Armv7L | Arch::X86 | Arch::Wasm32 => 32,
+            Arch::Aarch64
+            | Arch::Powerpc64
+            | Arch::Powerpc64Le
+            | Arch::X86_64
+            | Arch::S390X
+            | Arch::Riscv64
+            | Arch::Mips64el
+            | Arch::Sparc64 => 64,
+            Arch::Armv6L
+            | Arch::Armv7L
+            | Arch::X86
+            | Arch::Wasm32
+            | Arch::Mipsel
+            | Arch::Powerpc => 32,
         }
     }
 
-    /// Returns target triple string
+    /// Returns target triple as string
     pub fn target_triple(&self) -> &str {
         &self.triple
     }
 
+    /// Returns host triple as string
+    pub fn host_triple(&self) -> &str {
+        &self.rustc_version.host
+    }
+
     /// Returns true if the current platform is not windows
     pub fn is_unix(&self) -> bool {
         match self.os {
@@ -502,25 +566,32 @@ impl Target {
 
     /// Returns the path to the python executable inside a venv
     pub fn get_venv_python(&self, venv_base: impl AsRef<Path>) -> PathBuf {
-        if self.is_windows() {
-            let path = venv_base.as_ref().join("Scripts").join("python.exe");
-            if path.exists() {
-                path
-            } else {
-                // for conda environment
-                venv_base.as_ref().join("python.exe")
-            }
+        let python = if self.is_windows() {
+            "python.exe"
         } else {
-            venv_base.as_ref().join("bin").join("python")
-        }
+            "python"
+        };
+        self.get_venv_bin_dir(venv_base).join(python)
     }
 
     /// Returns the directory where the binaries are stored inside a venv
     pub fn get_venv_bin_dir(&self, venv_base: impl AsRef<Path>) -> PathBuf {
+        let venv = venv_base.as_ref();
         if self.is_windows() {
-            venv_base.as_ref().join("Scripts")
+            let bin_dir = venv.join("Scripts");
+            if bin_dir.exists() {
+                return bin_dir;
+            }
+            // Python innstalled via msys2 on Windows might produce a POSIX-like venv
+            // See https://github.com/PyO3/maturin/issues/1108
+            let bin_dir = venv.join("bin");
+            if bin_dir.exists() {
+                return bin_dir;
+            }
+            // for conda environment
+            venv.to_path_buf()
         } else {
-            venv_base.as_ref().join("bin")
+            venv.join("bin")
         }
     }
 
@@ -577,22 +648,20 @@ impl Target {
     }
 }
 
-pub(crate) fn get_host_target() -> Result<String> {
-    let host = rustc_version::version_meta()
-        .map(|meta| meta.host)
-        .map_err(|err| match err {
-            rustc_version::Error::CouldNotExecuteCommand(e)
-                if e.kind() == std::io::ErrorKind::NotFound =>
-            {
-                anyhow!(
-                    "rustc, the rust compiler, is not installed or not in PATH. \
+fn rustc_version_meta() -> Result<VersionMeta> {
+    let meta = rustc_version::version_meta().map_err(|err| match err {
+        rustc_version::Error::CouldNotExecuteCommand(e)
+            if e.kind() == std::io::ErrorKind::NotFound =>
+        {
+            anyhow!(
+                "rustc, the rust compiler, is not installed or not in PATH. \
                      This package requires Rust and Cargo to compile extensions. \
                      Install it through the system's package manager or via https://rustup.rs/.",
-                )
-            }
-            err => anyhow!(err).context("Failed to run rustc to get the host target"),
-        })?;
-    Ok(host)
+            )
+        }
+        err => anyhow!(err).context("Failed to run rustc to get the host target"),
+    })?;
+    Ok(meta)
 }
 
 fn macosx_deployment_target(
@@ -639,6 +708,7 @@ fn emcc_version() -> Result<String> {
 #[cfg(test)]
 mod test {
     use super::macosx_deployment_target;
+    use pretty_assertions::assert_eq;
 
     #[test]
     fn test_macosx_deployment_target() {
